/*
-------------------------------------------------------------------------------
File Name : example.c

Purpose : Example on how to read and write png files using the readpng files.
            Results in a new image file which is the original image rotated
            180 degrees. First for a gray scale image, and then for an RGB 
            image.

Creation Date : 2010-01-13

Last Modified : sø. 17. jan. 2010 kl. 13.30 +0100

Created By :  Martin Ertsås
-------------------------------------------------------------------------------
*/

#include "readpng.h"
#include <stdlib.h>

int main()
{
    double **image;
    int width, height, depth;
    char *name = "lena.png";

    //Reading a gray scale image.
    //Read image info.
    int test = read_png_info(name, &width, &height, &depth);

    if(test != 0)
        return test;

    //Allocate sufficient memory.
    image = malloc(height*sizeof(double*));
    int i,j;
    for(i = 0; i < height; i++) {
        image[i] = malloc(width*sizeof(double));
    }

    //Read the image.
    test = read_png_gray(image, width, height);

    if(test != 0) {
        for(i = 0; i < height; i++) {
            free(image[i]);
        }
        free(image);
        return test;
    }

    double **out;
    out = malloc(height*sizeof(double*));
    for(i = 0; i < height; i++) {
        out[i] = malloc(width*sizeof(double));
        for(j = 0; j < width; j++) {
            out[i][j] = image[height - i - 1][width - j - 1];
        }
    }

    //Write image back to file.
    test = write_png_gray("lena2.png", out, width, height, depth);

    //Clean up memory.
    for(i = 0; i < height; i++) {
        free(image[i]);
        free(out[i]);
    }
    free(image);
    free(out);

    //Reading an RGB image.
    name = "rose_rgb.png";
    double **red, **green, **blue;
    test = read_png_info(name, &width, &height, &depth);

    if(test != 0)
        return test;

    red = malloc(height*sizeof(double*));
    green = malloc(height*sizeof(double*));
    blue = malloc(height*sizeof(double*));
    for(i = 0; i < height; i++) {
        red[i] = malloc(width*sizeof(double));
        green[i] = malloc(width*sizeof(double));
        blue[i] = malloc(width*sizeof(double));
    }

    test = read_png_rgb(red, green, blue, width, height);

    if(test != 0) {
        for(i = 0; i < height; i++) {
            free(red[i]);
            free(green[i]);
            free(blue[i]);
        }
        free(red);
        free(green);
        free(blue);
    }

    double **r_out, **g_out, **b_out;
    r_out = malloc(height*sizeof(double*));
    g_out = malloc(height*sizeof(double*));
    b_out = malloc(height*sizeof(double*));
    for(i = 0; i < height; i++) {
        r_out[i] = malloc(width*sizeof(double));
        g_out[i] = malloc(width*sizeof(double));
        b_out[i] = malloc(width*sizeof(double));
        for(j = 0; j < width; j++) {
            r_out[i][j] = red[height - i - 1][width - j - 1];
            g_out[i][j] = green[height - i - 1][width - j - 1];
            b_out[i][j] = blue[height - i - 1][width - j - 1];
        }
    }

    write_png_rgb("rose_rgb2.png", r_out, g_out, b_out, width, height, depth);

    for(i = 0; i < height; i++) {
        free(red[i]);
        free(green[i]);
        free(blue[i]);
        free(r_out[i]);
        free(g_out[i]);
        free(b_out[i]);
    }
    free(red);
    free(green);
    free(blue);
    free(r_out);
    free(g_out);
    free(b_out);

    return test;
}
